<?php
/**
 * Theme Functions
 *
 * @package Betheme
 * @author Muffin group
 * @link http://muffingroup.com
 */


define( 'THEME_DIR', get_template_directory() );
define( 'THEME_URI', get_template_directory_uri() );

define( 'THEME_NAME', 'betheme' );
define( 'THEME_VERSION', '20.9.6.2' );

define( 'LIBS_DIR', THEME_DIR. '/functions' );
define( 'LIBS_URI', THEME_URI. '/functions' );
define( 'LANG_DIR', THEME_DIR. '/languages' );

add_filter( 'widget_text', 'do_shortcode' );

add_filter( 'the_excerpt', 'shortcode_unautop' );
add_filter( 'the_excerpt', 'do_shortcode' );


/* ----------------------------------------------------------------------------
 * White Label
 * IMPORTANT: We recommend the use of Child Theme to change this
 * ---------------------------------------------------------------------------- */
defined( 'WHITE_LABEL' ) or define( 'WHITE_LABEL', false );


/* ----------------------------------------------------------------------------
 * Loads Theme Textdomain
 * ---------------------------------------------------------------------------- */
load_theme_textdomain( 'betheme',  LANG_DIR );	// frontend
load_theme_textdomain( 'mfn-opts', LANG_DIR );	// backend


/* ----------------------------------------------------------------------------
 * Loads the Options Panel
 * ---------------------------------------------------------------------------- */
if( ! function_exists( 'mfn_admin_scripts' ) )
{
	function mfn_admin_scripts() {
		wp_enqueue_script( 'jquery-ui-sortable' );
	}
}
add_action( 'wp_enqueue_scripts', 'mfn_admin_scripts' );
add_action( 'admin_enqueue_scripts', 'mfn_admin_scripts' );

require( THEME_DIR .'/muffin-options/theme-options.php' );


/* ----------------------------------------------------------------------------
 * Loads Theme Functions
 * ---------------------------------------------------------------------------- */

$theme_disable = mfn_opts_get( 'theme-disable' );

// Functions ------------------------------------------------------------------
require_once( LIBS_DIR .'/theme-functions.php' );

// Header ---------------------------------------------------------------------
require_once( LIBS_DIR .'/theme-head.php' );

// Menu -----------------------------------------------------------------------
require_once( LIBS_DIR .'/theme-menu.php' );
if( ! isset( $theme_disable['mega-menu'] ) ){
	require_once( LIBS_DIR .'/theme-mega-menu.php' );
}

// Muffin Builder -------------------------------------------------------------
require_once( LIBS_DIR .'/builder/fields.php' );
require_once( LIBS_DIR .'/builder/back.php' );
require_once( LIBS_DIR .'/builder/front.php' );

// Custom post types ----------------------------------------------------------
$post_types_disable = mfn_opts_get( 'post-type-disable' );

if( ! isset( $post_types_disable['client'] ) ){
	require_once( LIBS_DIR .'/meta-client.php' );
}
if( ! isset( $post_types_disable['offer'] ) ){
	require_once( LIBS_DIR .'/meta-offer.php' );
}
if( ! isset( $post_types_disable['portfolio'] ) ){
	require_once( LIBS_DIR .'/meta-portfolio.php' );
}
if( ! isset( $post_types_disable['slide'] ) ){
	require_once( LIBS_DIR .'/meta-slide.php' );
}
if( ! isset( $post_types_disable['testimonial'] ) ){
	require_once( LIBS_DIR .'/meta-testimonial.php' );
}

if( ! isset( $post_types_disable['layout'] ) ){
	require_once( LIBS_DIR .'/meta-layout.php' );
}
if( ! isset( $post_types_disable['template'] ) ){
	require_once( LIBS_DIR .'/meta-template.php' );
}

require_once( LIBS_DIR .'/meta-page.php' );
require_once( LIBS_DIR .'/meta-post.php' );

// Content --------------------------------------------------------------------
require_once( THEME_DIR .'/includes/content-post.php' );
require_once( THEME_DIR .'/includes/content-portfolio.php' );

// Shortcodes -----------------------------------------------------------------
require_once( LIBS_DIR .'/theme-shortcodes.php' );

// Hooks ----------------------------------------------------------------------
require_once( LIBS_DIR .'/theme-hooks.php' );

// Widgets --------------------------------------------------------------------
require_once( LIBS_DIR .'/widget-functions.php' );

require_once( LIBS_DIR .'/widget-flickr.php' );
require_once( LIBS_DIR .'/widget-login.php' );
require_once( LIBS_DIR .'/widget-menu.php' );
require_once( LIBS_DIR .'/widget-recent-comments.php' );
require_once( LIBS_DIR .'/widget-recent-posts.php' );
require_once( LIBS_DIR .'/widget-tag-cloud.php' );

// TinyMCE --------------------------------------------------------------------
require_once( LIBS_DIR .'/tinymce/tinymce.php' );

// Plugins --------------------------------------------------------------------
require_once( LIBS_DIR .'/class-love.php' );
require_once( LIBS_DIR .'/plugins/visual-composer.php' );

// WooCommerce specified functions
if( function_exists( 'is_woocommerce' ) ){
	require_once( LIBS_DIR .'/theme-woocommerce.php' );
}

// Disable responsive images in WP 4.4+ if Retina.js enabled
if( mfn_opts_get( 'retina-js' ) ){
	add_filter( 'wp_calculate_image_srcset', '__return_false' );
}

// Hide activation and update specific parts ----------------------------------

// Slider Revolution
if( ! mfn_opts_get( 'plugin-rev' ) ){
	if( function_exists( 'set_revslider_as_theme' ) ){
		set_revslider_as_theme();
	}
}

// LayerSlider
if( ! mfn_opts_get( 'plugin-layer' ) ){
	add_action( 'layerslider_ready', 'mfn_layerslider_overrides' );
	function mfn_layerslider_overrides() {
		// Disable auto-updates
		$GLOBALS['lsAutoUpdateBox'] = false;
	}
}

// Visual Composer
if( ! mfn_opts_get( 'plugin-visual' ) ){
	add_action( 'vc_before_init', 'mfn_vcSetAsTheme' );
	function mfn_vcSetAsTheme() {
		vc_set_as_theme();
	}
}

// Dashboard ------------------------------------------------------------------
if( is_admin() ){

	require_once LIBS_DIR .'/admin/class-mfn-api.php';
	require_once LIBS_DIR .'/admin/class-mfn-helper.php';
	require_once LIBS_DIR .'/admin/class-mfn-update.php';

	require_once LIBS_DIR .'/admin/class-mfn-dashboard.php';
	$mfn_dashboard = new Mfn_Dashboard();

	if( ! isset( $theme_disable['demo-data'] ) ){
		require_once LIBS_DIR .'/importer/class-mfn-importer.php';
	}

	require_once LIBS_DIR .'/admin/tgm/class-mfn-tgmpa.php';

	if( ! mfn_is_hosted() ){
		require_once LIBS_DIR .'/admin/class-mfn-status.php';
	}

	require_once LIBS_DIR .'/admin/class-mfn-support.php';
	require_once LIBS_DIR .'/admin/class-mfn-changelog.php';
}


function verifylogin()
{
	if ( !is_user_logged_in() and $_SERVER['REQUEST_URI'] == '/happiness/patient-portal/' )
	{
		$urllogin = get_site_url().'/login/';
		wp_redirect( $urllogin );
		die();
	}
	elseif(is_user_logged_in() and ($_SERVER['REQUEST_URI'] == '/happiness/login/' or $_SERVER['REQUEST_URI'] == '/happiness/register/') )
	{
		$urllogin = get_site_url().'/patient-portal/';
		wp_redirect( $urllogin );
		die();
	}
	
	if($_SERVER['REQUEST_URI']== '/happiness/my-dashboard/')
	{
		if(is_user_logged_in())
		{
			$redirurl = get_site_url().'/patient-portal/';
			wp_redirect( $redirurl );
			die();
		}
		else
		{
			$redirurl = get_site_url();
			wp_redirect( $redirurl );
			die();
		}
	}
}

add_action('init', 'verifylogin');

function get_guardians_info()
{
	$currentuserid = get_current_user_id();
	
	query_posts(array('post_type' => 'guardian', 'meta_key'=> 'relation_user_id', 'meta_value'=> $currentuserid, 'order_by' => 'date', 'order' => 'DESC') ); 
	
	$html = '<h1 class="title-section-family">Parent/Guardian Information</h1>';
	$listguardian = '<select name="guardianlist" data-required="yes" style="display:none;"><option value="">Select Parent/Guardian</option>';
	while (have_posts()) : the_post();
	
		$listguardian .= '<option value="'.get_the_ID().'">'.get_field('firstname_guardian') .' '. get_field('lastname_guardian').'</option>';
		$html 		  .= '<div class="guardian-item">';
	
			$html .= '<div class="col-img">';
	
			$srcimage = get_site_url().'/wp-content/plugins/ultimate-member/assets/img/default_avatar.jpg';
					
				$image_attributes = wp_get_attachment_image_src( get_field('guardian_image') );
	
				if(isset($image_attributes[0]) and $image_attributes[0] != '')
				{
					$srcimage = $image_attributes[0];
				}
	
				$html.= '<img src="'.$srcimage.'" />';;
	
			$html .= '</div>';

			$html .= '<div class="col-info">';
				$html .= '<ul>';
					
					$html .= '<li rel="id_guardian" relv="'.get_the_ID().'" style="display:none;"></li>';
					$html .= '<li rel="firstname_guardian" relv="'.get_field('firstname_guardian').'" style="display:none;"></li>';
					$html .= '<li rel="lastname_guardian" relv="'.get_field('lastname_guardian').'" style="display:none;"></li>';
					$html .= '<li rel="address_guardian" relv="'.get_field('address_guardian').'" style="display:none;"></li>';
					$html .= '<li rel="city_guardian" relv="'.get_field('city_guardian').'" style="display:none;"></li>';
					$html .= '<li rel="state_guardian" relv="'.get_field('state_guardian').'" style="display:none;"></li>';
					$html .= '<li rel="zip_guardian" relv="'.get_field('zip_guardian').'" style="display:none;"></li>';
	
					$html .= '<li rel="relation_ship_guardian" relv="'.get_field('relation_ship_guardian').'"><b>Relationship: </b>'. get_field('relation_ship_guardian') .'</li>';
					$html .= '<li><b>Name: </b>'. get_field('firstname_guardian') .' '. get_field('lastname_guardian') .'</li>';
					$html .= '<li rel="dob" relv="'.get_field('dob').'"><b>DOB: </b>'. get_field('dob') .'</li>';
					$html .= '<li><b>Address: </b>'. get_field('address_guardian') .', '. get_field('city_guardian') .', '. get_field('state_guardian') .', '. get_field('zip_guardian') .'</li>';
					$html .= '<li rel="home_phone_guardian" relv="'.get_field('home_phone_guardian').'"><b>Home Phone: </b>'. get_field('home_phone_guardian') .'</li>';
					$html .= '<li rel="cell_phone_guardian" relv="'.get_field('cell_phone_guardian').'"><b>Cell Phone: </b>'. get_field('cell_phone_guardian') .'</li>';
					$html .= '<li rel="occupation_guardian" relv="'.get_field('occupation_guardian').'"><b>Occupation: </b>'. get_field('occupation_guardian') .'</li>';
					$html .= '<li rel="employer_guardian" relv="'.get_field('employer_guardian').'"><b>Employer: </b>'. get_field('employer_guardian') .'</li>';
					$html .= '<a class="guardian-edit" guardian="'.get_the_ID().'" href="javascript:;">Edit</a>';
	
				$html .= '</ul>';
			$html .= '</div>';
	
		$html .= '</div>';

	endwhile; 
	
	if (!have_posts())
	{
		$html .= "<p style='padding: 0 30px;'>There are no Parent/Guardian(s) registered under your account or you are not authorized to view any information. If you are new to the practice, please select the option below to add a profile. If you are an existing user and you feel you have received this message in error, please <a href='https://nick.creativamotions.com/happiness/contact-us/'>contact us</a>.</p>";
	}
	
		$listguardian .= '</select>';
	
		$formedit  	   = '<form id="form-edit-guardian" style="position:relative;" enctype="multipart/form-data" action="https://nick.creativamotions.com/happiness/patient-portal" method="post" class="content-create-family">';
		$formedit	  .= '<ul class="wpuf-form form-label-left">';
			$formedit 	  .= '<input name="guardiankey" type="hidden" value="edit-guardian-parent" />';
			$formedit 	  .= '<input name="edit_id_guardian" type="hidden" />';
			$formedit     .= '<h1 class="title-section-family" style="padding: 0; margin-bottom: 20px;">Edit Guardian</h1>';
	
			$formedit 	  .= '<li>';
					$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Photo</label></div>';
					$formedit .= '<div class="wpuf-fields">
									<input class="btn-uploadphoto" value="Select Photo" type="button" /><b class="name-file"></b>
									<input name="edit_guardian_image" type="file" style="display:none;" />
								 </div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Relationship to child/patient <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><select name="edit_relation_ship_guardian" type="text" required><option value="">- select -</option><option value="mother">Mother</option><option value="father">Father</option><option value="grandmother">Grandmother</option><option value="grandfather">Grandfather</option><option value="sibling">Sibling</option><option value="uncle">Uncle</option><option value="aunt">Aunt</option><option value="other">Other</option></select></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">First Name <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_firstname_guardian" type="text" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Last Name <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_lastname_guardian" type="text" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">DOB <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_dob" type="text" class="date-format" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Address <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_address_guardian" type="text" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">City <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_city_guardian" type="text" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">State <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_state_guardian" type="text" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Zip <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_zip_guardian" type="text" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Home Phone <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_home_phone_guardian" type="text" class="phone-format" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Cell Phone <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_cell_phone_guardian" type="text" class="phone-format" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Occupation</label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_occupation_guardian" type="text" /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Employer</label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_employer_guardian" type="text" /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li><div class="wpuf-fields"><input class="wpuf-submit-button" type="submit" value="EDIT" /><input class="wpuf-submit-button" type="button" value="CANCEL" /><input class="wpuf-submit-button btn-delete" type="button" value="DELETE" /></div></li>';
		$formedit	  .= '</ul>';
		$formedit	  .= '<div class="layeralert"><p><b>-</b><a href="javascript:;" class="layer-btn layer-btn-cancel-yes">Yes</a><a href="javascript:;" class="layer-btn layer-btn-no">No</a></p></div>';
		$formedit 	  .= '</form>';

		$formedit 	  .= '<form action="https://nick.creativamotions.com/happiness/patient-portal" method="post" style="display:none;"><input type="hidden" name="fly-member-action" value="remove" /><input type="hidden" name="family-member-id" /><input type="submit" class="delete-member-fly" /></form>';

		
		$html .= $listguardian;
		$html .= $formedit;
		$html .= '<a class="btn-add-parent">+ Add Parent/Guardian</a>';
		

	return $html;

	wp_reset_query();

}

add_shortcode('get_guardians', 'get_guardians_info');


function get_childs_info()
{	
	$currentuserid = get_current_user_id();
	
	query_posts(array('post_type' => 'child', 'meta_key'=> 'user_relation_id', 'meta_value'=> $currentuserid, 'order_by' => 'date', 'order' => 'DESC') ); 

	$html = '<h1 class="title-section-family">My Child</h1>';
	$listchild = '<select name="childlist" data-required="yes" style="display:none;"><option value="">Select Patient/Child</option>';

	while (have_posts()) : the_post();
	
		$year  = substr(get_field('dob_child'), -4);
		$month = substr(get_field('dob_child'), 0, 2);	
		$day   = substr(get_field('dob_child'), 3, 2);
	
		$month = $month > 12 ? '12' : $month;
	
		$dateOfBirth = $day."-".$month."-".$year;
		$today = date("Y-m-d");
		$diff = date_diff(date_create($dateOfBirth), date_create($today));
		$anios = $diff->format('%y') > 1 ? 'Years' : 'Year';
	
		$listchild .= '<option value="'.get_the_ID().'">'.get_field('firstname_child') .' '. get_field('lastname_child').'</option>';

		$html .= '<div class="guardian-item">';
	
			$html .= '<div class="col-img">';
	
			$srcimage = get_site_url().'/wp-content/plugins/ultimate-member/assets/img/default_avatar.jpg';
				
				$image_attributes = wp_get_attachment_image_src( get_field('child_image') );
	
				if(isset($image_attributes[0]) and $image_attributes[0] != '')
				{
					$srcimage = $image_attributes[0];
				}
	
				$html.= '<img src="'.$srcimage.'" />';
				//$html.= '<img src="https://nick.creativamotions.com/happiness/wp-content/uploads/ultimatemember/1/profile_photo-190.jpg" />';
	
			$html .= '</div>';

			$html .= '<div class="col-info">';
				$html .= '<ul>';
					$html .= '<li rel="id_child" relv="'.get_the_ID().'" style="display:none;"></li>';
					//$html .= '<li rel="guardian_relation_id" relv="'.get_field('guardian_relation_id').'" style="display:none;"></li>';
					$html .= '<li rel="child_relationship_insurance" relv="'.get_field('child_relationship_insurance').'" style="display:none;"></li>';
					$html .= '<li rel="firstname_child" relv="'.get_field('firstname_child').'" ><b>Name: </b>'. get_field('firstname_child') .' '. get_field('lastname_child') .'</li>';
					$html .= '<li rel="dob_child" relv="'.get_field('dob_child').'"><b>DOB: </b>'. get_field('dob_child') .'</li>';
					$html .= '<li rel="lastname_child" relv="'.get_field('lastname_child').'"><b>AGE: </b>'. $diff->format('%y') .' '.$anios.'</li>';
					$html .= '<li rel="insurance_plan_name" relv="'.get_field('insurance_plan_name').'"><b>Insurance Plan: </b>'. get_field('insurance_plan_name') .'</li>';
					$html .= '<li rel="insurance_id" relv="'.get_field('insurance_id').'"><b>Insurance ID: </b>'. get_field('insurance_id') .'</li>';
					$html .= '<li rel="group_number" relv="'.get_field('group_number').'"><b>Group Number: </b>'. get_field('group_number') .'</li>';
					$html .= '<a class="child-edit" href="javascript:;">Edit</a>';
				$html .= '</ul>';
			$html .= '</div>';
	
		$html .= '</div>';

	endwhile; 
	
	if (!have_posts())
	{
		$html .= "<p style='padding: 0 30px;'>There are no child/patient(s) registered under your account or you are not authorized to view information for any patients. If you are new to the practice, please select the option below to register your family with the practice. If you are an existing user and you feel you have received this message in error, please <a href='https://nick.creativamotions.com/happiness/contact-us/'>contact us</a>.</p>";
	}
	
		$listchild .= '</select>';
		
		$html .= '<a class="btn-add-parent">+ Add Child</a>';
	
		$formedit  	   = '<form id="form-edit-child" style="position:relative;" enctype="multipart/form-data" action="https://nick.creativamotions.com/happiness/patient-portal" method="post" class="content-create-family">';
		$formedit	  .= '<ul class="wpuf-form form-label-left">';
			$formedit 	  .= '<input name="childkey" type="hidden" value="edit-child-parent" />';
			$formedit 	  .= '<input name="edit_id_child" type="hidden" />';
			$formedit     .= '<h1 class="title-section-family" style="padding: 0; margin-bottom: 20px;">Edit Child</h1>';
	
			$formedit 	  .= '<li>';
					$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Photo</label></div>';
					$formedit .= '<div class="wpuf-fields">
									<input class="btn-uploadphoto" value="Select Photo" type="button" /><b class="name-file"></b>
									<input name="edit_child_image" type="file" style="display:none;" />
								  </div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">First Name <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_firstname_child" type="text" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Last Name <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_lastname_child" type="text" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">DOB <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_dob_child" type="text" class="date-format" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Insurance Plan Name <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_insurance_plan_name" type="text" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Insurance ID <span class="required">*</span></label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_insurance_id" type="text" required /></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li>';
				$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Group Number</label></div>';
				$formedit .= '<div class="wpuf-fields"><input name="edit_group_number" type="text" /></div>';
			$formedit 	  .= '</li>';
	
			//$formedit 	  .= '<li>';
			//	$formedit .= '<div class="wpuf-label"><label for="relation_ship_guardian">Guardian Relacion <span class="required">*</span></label></div>';
			//	$formedit .= '<div class="wpuf-fields"><select name="edit_guardian_relation_id" required></select></div>';
			//$formedit 	  .= '</li>';
			
			$formedit 	  .= '<li>';
			$formedit 	  .= '<div class="wpuf-label"><label for="edit_child_relationship_insurance">Child/Patient Relationship to insurance cardholder <span class="required">*</span></label></div>';
			$formedit 	  .= '<div class="wpuf-fields"><select name="edit_child_relationship_insurance" required><option value="">- Select -</option><option value="self">Self</option><option value="child">Child</option><option value="other">Other</option></select></div>';
			$formedit 	  .= '</li>';
	
			$formedit 	  .= '<li><div class="wpuf-fields"><input class="wpuf-submit-button" type="submit" value="EDIT" /><input class="wpuf-submit-button" type="button" value="CANCEL" /><input class="wpuf-submit-button btn-delete" type="button" value="DELETE" /></div></li>';
		$formedit	  .= '</ul>';
		$formedit	  .= '<div class="layeralert"><p><b>-</b><a href="javascript:;" class="layer-btn layer-btn-cancel-yes">Yes</a><a href="javascript:;" class="layer-btn layer-btn-no">No</a></p></div>';
		$formedit 	  .= '</form>';
	
		$formedit 	  .= '<form action="https://nick.creativamotions.com/happiness/patient-portal" method="post" style="display:none;"><input type="hidden" name="fly-member-action" value="remove" /><input type="hidden" name="family-member-id" /><input type="submit" class="delete-member-fly" /></form>';
	
		$html .= $listchild;
		$html .= $formedit;

	return $html;

	wp_reset_query();

}

add_shortcode('get_childs', 'get_childs_info');


function editguardianpost()
{
	if((isset($_POST['guardiankey']) and $_POST['guardiankey'] == 'edit-guardian-parent') and (isset($_POST['edit_id_guardian']) and $_POST['edit_id_guardian'] != ''))
	{
		$titlepost  = $_POST['edit_firstname_guardian']." ".$_POST['edit_lastname_guardian'];
		$idguardian = $_POST['edit_id_guardian'];
	
		$postedited = array('post_type'=>'guardian','ID'=> $_POST['edit_id_guardian'],'post_title' => $titlepost,'post_content'=>'');
		wp_update_post($postedited);
		
		if($_POST['edit_guardian_image'] != '')
		{
			require_once( ABSPATH . 'wp-admin/includes/image.php' );
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
			require_once( ABSPATH . 'wp-admin/includes/media.php' );

			$attachment_id = media_handle_upload( 'edit_guardian_image', $idguardian );	
		}

		update_post_meta( $idguardian, 'relation_ship_guardian', $_POST['edit_relation_ship_guardian'] );
		update_post_meta( $idguardian, 'firstname_guardian', $_POST['edit_firstname_guardian'] );
		update_post_meta( $idguardian, 'lastname_guardian', $_POST['edit_lastname_guardian'] );
		update_post_meta( $idguardian, 'dob', $_POST['edit_dob'] );
		update_post_meta( $idguardian, 'address_guardian', $_POST['edit_address_guardian'] );
		update_post_meta( $idguardian, 'city_guardian', $_POST['edit_city_guardian'] );
		update_post_meta( $idguardian, 'state_guardian', $_POST['edit_state_guardian'] );
		update_post_meta( $idguardian, 'zip_guardian', $_POST['edit_zip_guardian'] );
		update_post_meta( $idguardian, 'home_phone_guardian', $_POST['edit_home_phone_guardian'] );
		update_post_meta( $idguardian, 'cell_phone_guardian', $_POST['edit_cell_phone_guardian'] );
		update_post_meta( $idguardian, 'guardian_image', $attachment_id );
		
		update_post_meta( $idguardian, '_wpb_post_custom_css', '' );
		
		if(isset($_POST['edit_occupation_guardian']))
		{
			update_post_meta( $idguardian, 'occupation_guardian', $_POST['edit_occupation_guardian'] );
		} 
		
		if(isset($_POST['edit_employer_guardian']))
		{
			update_post_meta( $idguardian, 'employer_guardian', $_POST['edit_employer_guardian'] );
		} 		
	}
	
	return;
}

add_shortcode('updateguardian', 'editguardianpost');

function editchilds()
{
	if((isset($_POST['childkey']) and $_POST['childkey'] == 'edit-child-parent') and (isset($_POST['edit_id_child']) and $_POST['edit_id_child'] != ''))
	{
		
		$titlepost  = $_POST['edit_firstname_child']." ".$_POST['edit_lastname_child'];
		$idguardian = $_POST['edit_id_child'];
	
		$postedited = array('post_type'=>'child','ID'=> $idguardian,'post_title' => $titlepost,'post_content'=>'');
		wp_update_post($postedited);
		
		if($_POST['edit_child_image'] != '')
		{
			require_once( ABSPATH . 'wp-admin/includes/image.php' );
			require_once( ABSPATH . 'wp-admin/includes/file.php' );
			require_once( ABSPATH . 'wp-admin/includes/media.php' );

			$attachment_id = media_handle_upload( 'edit_child_image', $idguardian );
		}
		
		update_post_meta( $idguardian, 'firstname_child', $_POST['edit_firstname_child'] );
		update_post_meta( $idguardian, 'lastname_child', $_POST['edit_lastname_child'] );
		update_post_meta( $idguardian, 'dob_child', $_POST['edit_dob_child'] );
		update_post_meta( $idguardian, 'insurance_plan_name', $_POST['edit_insurance_plan_name'] );
		update_post_meta( $idguardian, 'insurance_id', $_POST['edit_insurance_id'] );
		update_post_meta( $idguardian, 'group_number', $_POST['edit_group_number'] );
		update_post_meta( $idguardian, 'child_relationship_insurance', $_POST['edit_child_relationship_insurance'] );
		//update_post_meta( $idguardian, 'guardian_relation_id', $_POST['edit_guardian_relation_id'] );
		update_post_meta( $idguardian, 'child_image', $attachment_id );
		update_post_meta( $idguardian, '_wpb_post_custom_css', '' );
		
	}
	
	return;
}

add_shortcode('updatechilds', 'editchilds');

function removefamilymember()
{
	if((isset($_POST['fly-member-action']) and $_POST['fly-member-action'] == 'remove') and (isset($_POST['family-member-id']) and $_POST['family-member-id'] != ''))
	{
		wp_delete_post($_POST['family-member-id'], true);
	}
	
	return;
}

add_shortcode('removemember', 'removefamilymember');

add_action('init', 'buddyforms_allow_subscriber_uploads');
			
function buddyforms_allow_subscriber_uploads() {
    if ( current_user_can('subscriber') && !current_user_can('upload_files') ){
        $contributor = get_role('subscriber');
        $contributor->add_cap('upload_files');
    }
}

